<?php
/**
 * OOPでBBS (Step3)
 *
 * オブジェクト指向プログラミングで掲示板を作ってみる。
 * ブログでその作り方を晒すために書いたものです。
 * 詳しい事は当該記事を見てください。 
 *
 *
 *           
 * PHP version 5.3
 *
 * LICENSE : このソースはCreative Commons 表示 - 非営利 2.1 日本 Licenseです。
 *           改変は自由ですが営利目的で利用することは出来ません。
 * 
 * @category  Step3
 * @package   OOPdeBBS
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 *
 */


/**
 * 掲示板のデータを操作するクラス
 *
 * ModelはControllerに使役されるクラスです。
 * 掲示板のデータに関する手続きを行います。
 *
 * <step3>
 * 丸投げされたデータを処理して格納しておく$_postと、
 * フォームのnameキーの設定を入れた$_inputs、ログファイル名とディレクトリを入れるメンバ変数を作成。
 * 投げられたデータは$_inputsを足がかりにして必要なものだけ抜き出し、セッターを使って$_postにセット。
 *　バリデーションや保存はゲッターで$_postの値を得て実行する。
 * 
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   Creative Commons BY-NC http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 */
class Model {
    
    const  DATA_DIRECTORY = 'dat/'; //ログのディレクトリ
    
    private $_inputs;
    private $_post;
    private $_logFilename;
    private $_error;
    
    public function __construct()
    {
        $this->_inputs = array('name', 'comment'); //フォームのname
        $this->_post = array();                    //$_POSTが入る
        
        $this->_logFilename = '';                  //ログファイル名
        $this->_error = NULL;                      //エラーが入る
    }
    
    /**
     * 仮のメッセージを表示する
     * @return {string} HelloWorldメッセージを返す。
     * @since 1.0 (step1)
     * @deprecated 1.3 (step3)
     */
    //public function dispatch()
    //{
    //    return 'Hello Model!';
    //}
    
    /**
     * 送信されたデータのセッター
     * @param {array} POST配列
     * @return {}
     * @since 1.3 (step3)
     */
    private function _setPost($data)
    {
        $this->_post = $data;
    }
    
    /**
     * 送信されたデータのゲッター
     * @return {array}
     * @since 1.3 (step3)
     */
    private function _getPost()
    {
        return $this->_post;
    }
    
    /**
     * エラーのセッター
     * @return {array}
     * @since 1.3 (step3)
     */
    public function setError($error)
    {
        $this->_error = $error;
    }
    /**
     * エラーのゲッター
     * @return {array}
     * @since 1.3 (step3)
     */
    public function getError()
    {
        return $this->_error;
    }
    
    /**
     * ログファイル名のセッター
     * @param {string} $name ファイル名（拡張子なし）
     * @return {}
     * @since 1.3 (step3)
     */
    public function setLogfileName($name)
    {
        $this->_logFilename = $name . '.dat';
    }
    /**
     * ログファイル名のゲッター
     * @return {array}
     * @since 1.3 (step3)
     */
    private function _getLogfileName()
    {
        return self::DATA_DIRECTORY . $this->_logFilename;
    }
    
    /**
     * 送信された内容をメンバ変数にセット
     * @param {array} $posts $_POST配列
     * @return {array} エラーメッセージの配列
     * @since 1.3 (step3)
     */
    public function setPostdata($posts)
    {
        foreach($this->_inputs as $target){
            $data[$target] = $posts[$target];
        }
        
        $this->_setPost($data);//セッターに渡す
        
        return;
    }
    
    /**
     * データのバリデーション
     * 
     * 入力されているかどうか確認。
     * 
     * @param {array} POST配列
     * @return {array} エラー配列 
     * @since 1.3 (step3)
     */
    public function validation()
    {
        $data = $this->_getPost();
        $error = array();
        
        foreach($data as $key => $value){
            if($key =='name' && $value === ''){
                $error[] = '名前を入力してください';
                
            }elseif($key=='comment' && $value === ''){
                $error[] = 'コメントを入力してください';
                
            }
        }
        
        $this->setError($error);
        
        if(count($error)>0){
            return false;
        }else{
            return true;
        }
    }
    
    /**
     * データの保存
     * 
     * csv形式でログファイルに書き込む。
     * 
     * @return {}
     * @since 1.3 (step3)
     */
    public function write()
    {
        $data = $this->_getPost();
        
        $data = array_map(array($this,'_clean'), $data);//クリーンアップ
        
        array_unshift($data, time());//先頭にタイムスタンプ
        
        $log = file_get_contents($this->_getLogfileName());//ログ確保

        $fp = fopen($this->_getLogfileName(), 'w');
    
        $this->myFputcsv($fp, $data);
        fwrite($fp, trim($log));
        
        fclose($fp);
        
        return;
    }
    
    /**
     * 値をクリーンアップする
     * @param {string} $str フォームで入力された文字列
     * @param {string} タグとか除去した文字列
     * @since 1.3 (step3)
     */
    private function _clean($str)
    {
        $str = strip_tags(trim($str));
        $str = preg_replace('/<(script|style).*?<\/(script|style)>/sm', '', $str);//script
        $str = preg_replace("/<\!--.*-->/sm", '', $str); //comments
        $str = htmlentities($str,ENT_QUOTES,"UTF-8");
        $str = str_replace(array("\r\n", "\r", "\n\n", "\n", "\0"), "", nl2br($str));
        $str = str_replace(array("  ", ",", '"'), array("\t", '&#044;', '&quot;'), $str);
        
        if (get_magic_quotes_gpc()) {
            $str = stripslashes($str);
        }
        
        return $str;
        
    }
    
    /**
     * エンクロージャー問題解決用のfputcsv
     *
     * 日本語文字列が囲まれてないとfgetcsvしたとき文字化けする。
     * オリジナルのfputcsvはエスケープ文字が含まれているときしか囲ってくれないから
     * 半角英数以外の文字が含まれていたら$enclosureで囲むようにする。
     * マニュアルのコメントから拝借したソースをカスタマイズ。
     * 
     * @param {fp} $handle ファイルポインタ
     * @param {array} $fields 値の配列
     * @param {string} $delimiter フィールド区切り文字 
     * @param {string} $enclosure フィールドを囲む文字
     * @return {numeric, false} 書き込んだバイト数、またはエラー時に FALSE（fwriteと同じ）
     * @since 1.3 (step3)
     */
    private function myFputcsv($handle, $fields, $delimiter=',', $enclosure='"', $mysql_null = false)
    {
        $delimiter_esc = preg_quote($delimiter, '/');
        $enclosure_esc = preg_quote($enclosure, '/');
    
        $output = array();
        foreach ($fields as $field) {
            if ($field === null && $mysql_null) {
                $output[] = 'NULL';
                continue;
            }
    
            $output[] = preg_match("/(?:${delimiter_esc}|${enclosure_esc}|\s|[^\x01-\x7e])/", $field) 
                        ? ($enclosure . str_replace($enclosure, $enclosure . $enclosure, $field) . $enclosure)
                        : $field;
        }
    
        return fwrite($handle, join($delimiter, $output) . "\n"); 
    }
}

?>